﻿//Copyright (C) 2007-2024 Mariano Ortu <https://www.speedcrypt.info/>

using System;
using System.Xml;
using System.Data;
using System.Linq;
using System.Xml.Linq;
using System.Windows.Forms;
using System.Drawing;
using System.Collections.Generic;
using System.IO;


namespace XMLConfig
{
    public partial class FrmMain : Form
    {
        #region Fields

        private string CFG = string.Empty;
        private PrivateXmlConfig _xmlConfig = new PrivateXmlConfig();
        private int flg = 0;//Switch
        //String List
        List<string> _RECLIST = new List<string>();

        #endregion Fields

        #region Constructors
        public FrmMain()
        {
            InitializeComponent();
            loadall();

        }

        #endregion Contructors

        #region Settings
        void loadall()
        {
            //Form Settings
            Text = "XML Config Example...";
            StartPosition = FormStartPosition.CenterScreen;
            MaximizeBox = false;
            MinimizeBox = false;
            FormBorderStyle = FormBorderStyle.FixedDialog;

            ///Configuration File
            CFG = System.Reflection.Assembly.GetExecutingAssembly().Location;
            if (CFG.EndsWith(".exe"))
                CFG = CFG.Substring(0, CFG.Length - 4);
            CFG += ".config.xml";
            _xmlConfig = new PrivateXmlConfig();
            _xmlConfig.LoadFromFile(CFG);

            //Menu
            infoMenuItem.Click += InfoMenuItem_Click;
            closeMenuItem.Click += CloseMenuItem_Click;

            //ListView
            LstV.View = View.Details;
            LstV.BackColor = Color.LightGray;
            LstV.ForeColor = Color.DarkBlue;
            LstV.BeginUpdate();
            LstV.Columns.Add("ONE", 130, HorizontalAlignment.Center);
            LstV.Columns.Add("TWO", 130, HorizontalAlignment.Left);
            LstV.Columns.Add("THREE", 130, HorizontalAlignment.Left);
            LstV.Columns.Add("CNT", 40, HorizontalAlignment.Left);
            LstV.EndUpdate();
            LstV.Font = new Font("Courier New", 9.25F, FontStyle.Regular);
            LstV.MultiSelect = false;
            LstV.SelectedIndexChanged += LstV_SelectedIndexChanged;
            LstV.MouseUp += LstV_MouseUp;
            LstV.MouseMove += LstV_MouseMove;

            //Buttons
            btnClr.Enabled = false;
            btnClr.Click += BtnClr_Click;
            btnAdd.Enabled = false;
            btnAdd.Click += BtnAdd_Click;
            btnMod.Enabled = false;
            btnMod.Click += BtnMod_Click;
            btnRem.Enabled = false;
            btnRem.Click += BtnRem_Click;
            btnKeyv.Enabled = false;
            btnKeyv.Click += BtnKeyv_Click;
            btnlist.Enabled = false;
            btnlist.Click += Btnlist_Click;
            btnClr.Cursor = Cursors.Hand;
            btnAdd.Cursor = btnClr.Cursor;
            btnMod.Cursor = btnClr.Cursor;
            btnRem.Cursor = btnClr.Cursor;
            btnKeyv.Cursor = btnClr.Cursor;
            btnlist.Cursor = btnClr.Cursor;

            //TextBox
            txtF.TextChanged += TxtF_TextChanged;
            txtS.TextChanged += TxtS_TextChanged;
            txtT.TextChanged += TxtT_TextChanged;

            if (File.Exists(CFG))
            {
                Loadrec();
                btnlist.Enabled = true;
            }
        }

        #endregion Settings

        #region Menu
        private void CloseMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }
        private void InfoMenuItem_Click(object sender, EventArgs e)
        {
            FrmInfo frm = new FrmInfo(this)
            {
                StartPosition = FormStartPosition.CenterScreen
            };
            frm.ShowDialog();
        }

        #endregion Menu

        #region ListView
        private void LstV_MouseUp(object sender, MouseEventArgs e)
        {
            if (LstV.Items.Count > 0)
            {
                btnMod.Enabled = false;
                btnRem.Enabled = btnMod.Enabled;
                btnKeyv.Enabled = btnMod.Enabled;
                foreach (ListViewItem selitem in LstV.SelectedItems)
                {
                    btnMod.Enabled = selitem.Selected == true;
                    btnRem.Enabled = btnMod.Enabled;
                    btnKeyv.Enabled = btnMod.Enabled;
                }
            }
        }
        private void LstV_MouseMove(object sender, MouseEventArgs e)
        {
            if (LstV.HitTest(e.X, e.Y).Item != null)
                Cursor = Cursors.Hand;
            else
            {
                Cursor = Cursors.Default;
            }
        }
        private void LstV_SelectedIndexChanged(object sender, EventArgs e)
        {
            flg = 1;//Switch
            foreach (ListViewItem item in LstV.Items)
            {
                if (item.Selected == true)
                {
                    txtF.Text = item.SubItems[0].Text;
                    txtS.Text = item.SubItems[1].Text;
                    txtT.Text = item.SubItems[2].Text;
                    txtSng.Text = item.SubItems[3].Text;
                }
            }
            btnMod.Enabled = txtF.Text != string.Empty && txtS.Text != string.Empty && txtT.Text != string.Empty;
            btnRem.Enabled = true;
            btnKeyv.Enabled = true;
            flg = 0;//Switch Default
        }
        void Loadrec()
        {
            string valuelist, strKeyValue;
            strKeyValue = _xmlConfig.GetValue("Results.Counter");
            txtCnt.Text = strKeyValue;
            _RECLIST.Clear();//Resets the List
            //Create the XmlDocument.
            XmlDocument doc = new XmlDocument();
            doc.Load(CFG);
            XmlNodeList elemList = doc.GetElementsByTagName("Key");
            for (int i = 0; i < elemList.Count; i++)
            {
                if (!elemList[i].Attributes["Name"].Value.Contains("Counter"))
                   _RECLIST.Add(elemList[i].Attributes["Value"].Value);
            }
            LstV.Items.Clear();
            List<string> list = _RECLIST.Cast<string>().ToList();
            for (int a = 0; a < list.Count; a++)
            {
                valuelist = list.ElementAt(a);
                char delimiter = '|';
                string[] substrings = valuelist.Split(delimiter);
                ListViewItem item = new ListViewItem()
                {
                    Text = substrings[0]
                };
                item.SubItems.Add(substrings[1]);
                item.SubItems.Add(substrings[2]);
                item.SubItems.Add(substrings[3]);
                LstV.Items.Add(item);
            }
        }

        #endregion ListView

        #region Buttons
        private void BtnClr_Click(object sender, EventArgs e)
        {
            clrtxt();
            btnMod.Enabled = false;
            btnRem.Enabled = btnMod.Enabled;
            btnKeyv.Enabled = btnMod.Enabled;
            txtF.Focus();
        }
        private void BtnAdd_Click(object sender, EventArgs e)
        {
            try
            {
                _xmlConfig = new PrivateXmlConfig();
                _xmlConfig.LoadFromFile(CFG);
                int cnt = int.Parse(txtCnt.Text);
                cnt = cnt + 1;
                txtCnt.Text = cnt.ToString();
                //String Values
                string[] newsett = new string[] { txtF.Text, txtS.Text, txtT.Text, cnt.ToString() };
                _xmlConfig.SetValue("Results.Counter", txtCnt.Text);
                _xmlConfig.SetValue("Results.Testo" + cnt, string.Join("|", newsett));
            }
            finally
            {
                btnAdd.Enabled = false;
                btnlist.Enabled = true;
                clrtxt();
                txtF.Focus();
                _xmlConfig.SaveToFile(CFG);
                Loadrec();
            }
        }
        private void BtnMod_Click(object sender, EventArgs e)
        {
            try
            {
                string[] newsett = new string[] { txtF.Text, txtS.Text, txtT.Text, txtSng.Text };
                XDocument doc = XDocument.Load(CFG);
                XElement alt = doc.Descendants("Key").Where(x => (string)x.Attribute("Name") == "Results.Testo" + txtSng.Text).FirstOrDefault();
                alt.Attribute("Value").Value = string.Join("|", newsett);
                doc.Save(CFG);
            }
            finally
            {
                Loadrec();
                btnMod.Enabled = false;
                btnRem.Enabled = btnMod.Enabled;
                btnKeyv.Enabled = btnMod.Enabled;
                clrtxt();
            }
        }
        private void BtnRem_Click(object sender, EventArgs e)
        {
            try
            {
                XDocument doc = XDocument.Load(CFG);
                var q = from node in doc.Descendants("Key")
                        let attr = node.Attribute("Name")
                        where attr != null && attr.Value == "Results.Testo" + txtSng.Text
                        select node;
                q.ToList().ForEach(x => x.Remove());
                doc.Save(CFG);
                Loadrec();

                if (LstV.Items.Count == 0)
                {
                    //Reset the counter
                    XElement alt = doc.Descendants("Key").Where(x => (string)x.Attribute("Name") == "Results.Counter").FirstOrDefault();
                    alt.Attribute("Value").Value = "0";
                    doc.Save(CFG);
                    txtCnt.Text = "0";
                    txtSng.Text = txtCnt.Text;
                }
            }
            finally 
            {
                btnRem.Enabled = false;
                btnMod.Enabled = btnRem.Enabled;
                btnKeyv.Enabled = btnRem.Enabled;
                clrtxt();
            }
        }
        private void BtnKeyv_Click(object sender, EventArgs e)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(CFG);
            XmlNodeList elemList = doc.GetElementsByTagName("Key");
            for (int i = 0; i < elemList.Count; i++)
            {
                string str1 = elemList[i].Attributes["Value"].Value;
                if (str1.IndexOf(txtF.Text) != -1)
                    MessageBox.Show(str1);
            }
        }
        private void Btnlist_Click(object sender, EventArgs e)
        {
            var xmlFile = CFG;
            var doc = new XmlDocument();
            doc.Load(xmlFile);
            XmlElement root = doc.DocumentElement;
            int? n = root?.ChildNodes.Count;
            MessageBox.Show($"There are {n} elements");
        }

        #endregion Buttons

        #region TextBox
        private void TxtF_TextChanged(object sender, EventArgs e)
        {
            btnClr.Enabled = txtF.Text != string.Empty;
            if (txtF.TextLength > 0)
            {
                txtF.Text = txtF.Text.First().ToString().ToUpper() + String.Join("", txtF.Text.Skip(1)).ToLower();
                txtF.SelectionStart = txtF.TextLength;
            }
        }
        private void TxtS_TextChanged(object sender, EventArgs e)
        {
            if (txtS.TextLength > 0)
            {
                txtS.Text = txtS.Text.First().ToString().ToUpper() + String.Join("", txtS.Text.Skip(1)).ToLower();
                txtS.SelectionStart = txtS.TextLength;
            }
        }
        private void TxtT_TextChanged(object sender, EventArgs e)
        {
            if (flg == 0)
                btnAdd.Enabled = txtF.Text != string.Empty && txtS.Text != string.Empty && txtT.Text != string.Empty;
            if (txtT.TextLength > 0)
            {
                txtT.Text = txtT.Text.First().ToString().ToUpper() + String.Join("", txtT.Text.Skip(1)).ToLower();
                txtT.SelectionStart = txtT.TextLength;
            }
        }
        void clrtxt()
        {
            txtF.Text = string.Empty;
            txtS.Text = txtF.Text;
            txtT.Text = txtF.Text;
        }
        #endregion TextBox
    }
}
